package cz.muni.pa165.genre;

import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.info.Contact;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.models.security.SecurityScheme;
import org.springdoc.core.customizers.OpenApiCustomizer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configurers.oauth2.server.resource.OAuth2ResourceServerConfigurer;
import org.springframework.security.web.SecurityFilterChain;

@OpenAPIDefinition(
    info = @Info(
        title = "Movie genre microservice",
        version = "1.0",
        description = """
                        Simple service for genre manipulation. The API has operations for:
                        - getting all genres (with pagination)
                        - creating a new genre
                        - getting a specific genre by its id
                        """,
        contact = @Contact(name = "Michal Drobňák", email = "493239@muni.cz")
    )
)
@Tag(name = "Genre", description = "Microservice for movie genres") // metadata for inclusion into OpenAPI document
@ComponentScan(basePackages = "cz.muni.pa165")
@EnableJpaRepositories
@SpringBootApplication
public class GenreMicroservice {

    private static final String SECURITY_SCHEME_BEARER = "Bearer";
    public static final String SECURITY_SCHEME_NAME = SECURITY_SCHEME_BEARER;

    @Value("${rest.path.genres}/**")
    private String restApiPath;

    public static void main(String[] args) {
        SpringApplication.run(GenreMicroservice.class, args);
    }

    @Bean
    SecurityFilterChain securityFilterChain(HttpSecurity http)
        throws Exception {
        http
            .authorizeHttpRequests(x ->
                x
                    .requestMatchers(HttpMethod.GET, restApiPath)
                    .hasAuthority("SCOPE_test_read")
                    .requestMatchers(HttpMethod.POST, restApiPath)
                    .hasAuthority("SCOPE_test_write")
                    .requestMatchers(HttpMethod.PUT, restApiPath)
                    .hasAuthority("SCOPE_test_write")
                    .requestMatchers(HttpMethod.DELETE, restApiPath)
                    .hasAuthority("SCOPE_test_write")
                    .anyRequest()
                    .permitAll()
            )
            .oauth2ResourceServer(OAuth2ResourceServerConfigurer::opaqueToken);
        return http.build();
    }

    @Bean
    public OpenApiCustomizer openAPICustomizer() {
        return openApi -> {
            openApi
                .getComponents()
                .addSecuritySchemes(
                    SECURITY_SCHEME_BEARER,
                    new SecurityScheme()
                        .type(SecurityScheme.Type.HTTP)
                        .scheme("bearer")
                        .description("provide a valid access token")
                );
        };
    }
}
